import { defer, throwError, EMPTY, Observable } from 'rxjs';
import { finalize, catchError, scan, distinctUntilChanged } from 'rxjs/operators';
import { HttpErrorResponse, HttpEventType, HttpResponseBase } from '@angular/common/http';

/**
 * Invokes a callback upon subscription.
 *
 * @param callback function to invoke upon subscription
 * @returns stream which will invoke callback
 */
function prepare(callback) {
    return (source) => defer(() => {
        callback();
        return source;
    });
}

/**
 * Indicates whether the observable is currently loading (meaning subscription is active and
 * it hasn't completed or errored).
 *
 * @param indicator subject as target for indication
 * @returns stream which will indicate loading through passed subject
 */
function indicate(indicator) {
    return (source) => source.pipe(prepare(() => indicator.next(true)), finalize(() => indicator.next(false)));
}

/**
 * Maps Angular HTTP status codes to more semantic errors.
 *
 * @param codeErrors mapping from status code to error
 * @returns stream which will map http error codes to passed errors
 */
function throwForCodes(codeErrors) {
    return (source) => source.pipe(catchError(error => {
        if (error instanceof HttpErrorResponse) {
            return throwError(codeErrors[error.status]?.() ?? error);
        }
        return throwError(error);
    }));
}

function isHttpResponse(event) {
    return event.type === HttpEventType.Response;
}
function isHttpProgressEvent(event) {
    return (event.type === HttpEventType.DownloadProgress ||
        event.type === HttpEventType.UploadProgress);
}

function download(saver) {
    return (source) => source.pipe(scan((last, event) => {
        if (isHttpProgressEvent(event)) {
            return {
                progress: event.total
                    ? Math.round((100 * event.loaded) / event.total)
                    : last.progress,
                state: "IN_PROGRESS",
                content: null
            };
        }
        if (isHttpResponse(event)) {
            if (saver && event.body) {
                saver(event.body);
            }
            return {
                progress: 100,
                state: "DONE",
                content: event.body
            };
        }
        return last;
    }, { state: "PENDING", progress: 0, content: null }), distinctUntilChanged((a, b) => a.state === b.state &&
        a.progress === b.progress &&
        a.content === b.content));
}

/**
 * If the underlying request throws with a 404 error, it will swallow the error
 * and complete the observable instead. All other errors will be rethrown.
 *
 * You can then set a fallback value with `defaultIfEmpty`.
 *
 * @returns stream which will map 404 errors to `undefined`
 * @example
 * this.http.get<User>('/users/123').pipe(
 *   ignoreNotFound()
 * )
 */
function ignoreNotFound() {
    return (source) => source.pipe(catchError(error => {
        if (error instanceof HttpResponseBase && error.status === 404) {
            return EMPTY;
        }
        return throwError(error);
    }));
}

const noZoneRunner = (zone) => (source) => {
    return new Observable(subscriber => {
        return zone.runOutsideAngular(() => {
            runInZone(zone)(source).subscribe(subscriber);
        });
    });
};
function runOutsideZone(zone) {
    return source => {
        return new Observable(subscriber => {
            return source.subscribe((value) => zone.runOutsideAngular(() => subscriber.next(value)), (e) => zone.runOutsideAngular(() => subscriber.error(e)), () => zone.runOutsideAngular(() => subscriber.complete()));
        });
    };
}
function runInZone(zone) {
    return source => {
        return new Observable(subscriber => {
            return source.subscribe((value) => zone.run(() => subscriber.next(value)), (e) => zone.run(() => subscriber.error(e)), () => zone.run(() => subscriber.complete()));
        });
    };
}

function upload() {
    const initialState = { state: "PENDING", progress: 0, body: null };
    const reduceState = (state, event) => {
        if (isHttpProgressEvent(event)) {
            return {
                progress: event.total
                    ? Math.round((100 * event.loaded) / event.total)
                    : state.progress,
                state: "IN_PROGRESS",
                body: null,
            };
        }
        if (isHttpResponse(event)) {
            return {
                progress: 100,
                state: "DONE",
                body: event.body,
            };
        }
        return state;
    };
    return (source) => source.pipe(scan(reduceState, initialState), distinctUntilChanged((a, b) => a.state === b.state && a.progress === b.progress));
}

/*
 * Public API Surface of ngx-operators
 */

/**
 * Generated bundle index. Do not edit.
 */

export { download, ignoreNotFound, indicate, noZoneRunner, prepare, runInZone, runOutsideZone, throwForCodes, upload };
//# sourceMappingURL=ngx-operators.mjs.map
